package com.casper.sdk.getdictionary

import com.casper.sdk.ConstValues
import net.jemzart.jsonkraken.get
import net.jemzart.jsonkraken.toJson
import net.jemzart.jsonkraken.values.JsonObject
import java.io.BufferedReader
import java.io.InputStreamReader
import java.net.HttpURLConnection
import java.net.URL

/** Class built for state_get_dictionary_item RPC call */
class GetDictionaryItemRPC {
    var methodURL: String = ConstValues.TESTNET_URL
    /**
     * This function initiate the process of sending POST request with given parameter in JSON string format
     * The input parameterStr is used to send to server as parameter of the POST request to get the result back.
     * The input parameterStr is somehow like this:
     * {"method": "state_get_dictionary_item", "id": 1, "params": {"state_root_hash": "146b860f82359ced6e801cbad31015b5a9f9eb147ab2a449fd5cdb950e961ca8", "dictionary_identifier": {"AccountNamedKey": {"dictionary_name": "dict_name", "key": "account-hash-ad7e091267d82c3b9ed1987cb780a005a550e6b3d1ca333b743e2dba70680877", "dictionary_item_key": "abc_name"}}}, "jsonrpc" :  "2.0"}
     * The parameterStr is generated by the GetDictionaryItemParams class, declared in file GetDictionaryItemParams.kotlin
     * Then the GetDictionaryItemResult is retrieved by parsing JsonObject result
     * If the result is error,  then an exception is thrown
     * Else the GetDictionaryItemResult is taken by parsing the  retrieving JsonObject
     */
    @Throws(IllegalArgumentException:: class)
    fun getDictionaryItem(parameterStr: String):  GetDictionaryItemResult {
        val url = URL(methodURL)
        val con: HttpURLConnection = url.openConnection() as HttpURLConnection
        con.setRequestMethod("POST")
        con.setRequestProperty("Content-Type", "application/json")
        con.setRequestProperty("Accept", "application/json");
        con.doOutput = true
        con.outputStream.use { os ->
            val input: ByteArray = parameterStr.toByteArray()
            os.write(input, 0, input.size)
        }
        BufferedReader(
            InputStreamReader(con.inputStream, "utf-8")
        ).use {
            val response = StringBuilder()
            var responseLine: String?
            while (it.readLine().also { responseLine = it } != null) {
                response.append(responseLine!!.trim { it <= ' ' })
            }
            val json = response.toString().toJson()
            if(json.get("error") != null) {
                throw IllegalArgumentException("Error get state root hash")
            } else {
                return GetDictionaryItemResult.fromJsonObjectToGetDictionaryItemResult(json.get("result") as JsonObject)
            }
        }
    }
}